package MT::Plugin::GooglePageRank;
;#	MTGooglePageRank plugin for MovableType
;#			Original Copyright (c) 2006 Piroli YUKARINOMIYA
;#			Open MagicVox.net - http://www.magicvox.net/
;#			@see http://www.magicvox.net/archive/2006/04151455.php

use strict;

use vars qw( $MYNAME $VERSION );
$MYNAME = 'GooglePageRank';
$VERSION = '0.90';

use base qw( MT::Plugin );
my $plugin = new MT::Plugin ({
		name => $MYNAME,
		version => $VERSION,
		author_name => 'Piroli YUKARINOMIYA',
		author_link => 'http://www.magicvox.net/',
		doc_link => 'http://www.magicvox.net/',
		config_link => 'db-update.cgi',
		description => <<HTMLHEREDOC,
Enable to retrieve Google PageRank of any URL,
and deliver some tags for sorting items according to their PageRank.
HTMLHEREDOC
});
MT->add_plugin ($plugin);

sub instance { $plugin }


my $db_path = '/home/pass/to/your/cache/file/db-index.txt';


;########################################################################

use MT::Template::Context;
MT::Template::Context->add_tag (UseGooglePageRank => \&use_google_pagerank);
MT::Template::Context->add_container_tag (GetGooglePageRank => \&get_google_pagerank);
MT::Template::Context->add_container_tag (GPRPrepareContent => \&prepare_content);
MT::Template::Context->add_container_tag (GPRPrepareContentParam => \&prepare_content_param);
MT::Template::Context->add_tag (GPRContentLoop => \&content_loop);


;### Embed a PHP code to setup the MTGooglePageRank
sub use_google_pagerank {
	my ($ctx, $args) = @_;
;#
	my $php_code = <<'PHPHEREDOC';
<?php
// Read a whole of list into $pagerank_db
$mtgpr_pagerank_db = null;
$fp = @fopen ('%%INDEX_DB%%', 'r');
if ($fp) {
	if (flock ($fp, LOCK_SH | LOCK_NB)) {
		$mtgpr_pagerank_db = array ();
		while (! feof ($fp) && ($buf = fgets ($fp, 1024))) {
			list ($url, $t, $pr) = explode ("\t", trim ($buf));
			if (isset ($url))
				$mtgpr_pagerank_db[$url] = isset ($pr) ? $pr : -1;
		}
		flock ($fp, LOCK_UN);
	}
	fclose ($fp);
}

function mtgpr_GetGooglePageRank ($url) {
	global $mtgpr_pagerank_db;
	if ($mtgpr_pagerank_db)
	{
		// URL in indexes
		if (array_key_exists ($url, $mtgpr_pagerank_db)) {
			$rank = $mtgpr_pagerank_db[$url];
			return -1 < $rank ? $rank : '';
		}

		// Unknown URL? so adding as a que
		$fp = @fopen ('%%INDEX_DB%%', 'a');
		if ($fp) {
			if (flock ($fp, LOCK_EX | LOCK_NB)) {
				fwrite ($fp, $url. "\n");
				flock ($fp, LOCK_UN);
			}
			fclose ($fp);
		}
		$mtgpr_pagerank_db[$url] = -1;
	}
	return '';
}
?>
PHPHEREDOC

	;# Rewrite the path of the data file in PHP code
	$php_code =~ s/%%INDEX_DB%%/$db_path/g;
	$php_code;
}


;### Embed a small PHP code to retrieve the Google PageRank of URL
sub get_google_pagerank {
	my ($ctx, $args, $cond) = @_;
;#
	my $builder = $ctx->stash('builder');
	my $tokens  = $ctx->stash('tokens');
	defined (my $out = $builder->build ($ctx, $tokens, $cond))
		or return $ctx->error ($builder->errstr);

	$out =~ s/^\s+//;
	$out =~ s/\s+$//;

	'<?php echo mtgpr_GetGooglePageRank(\''. $out. '\'); ?>'. "\n";
}


;### Embed PHP codes to enclose contens preparing for MTGPRContentLoop
sub prepare_content {
	my ($ctx, $args, $cond) = @_;
;#
	;# retrieve arguments - name
	my $name = $args->{name} || '';
	;# retrieve param - url
	my $url = $ctx->stash (__PACKAGE__. '_url')
		or return $ctx->error ("<MTGPRPrepareContentParam> must be used once at least.");

	;# first loop of MTGPRUtilPrepareContent named $name
	my $output = '';
	my $count = $ctx->stash (__PACKAGE__. '::'. $name);
	if (! $count) {
		$count = 0;
		$output .= <<"PHPHEREDOC";
<?php \$mtgpr_content_$name = array (); ?>
PHPHEREDOC
	}
	$ctx->stash (__PACKAGE__. '::'. $name, ++$count);

	;# build contents
	my $builder = $ctx->stash('builder');
	my $tokens  = $ctx->stash('tokens');
	defined (my $out = $builder->build ($ctx, $tokens, $cond))
		or return $ctx->error ($builder->errstr);

	;# enclose contents with PHP codes
	$output .= <<"PHPHEREDOC";
<?php function mtgpr_content_$name$count () { ?>
$out
<?php }
array_push (\$mtgpr_content_$name, array (
	'content' => mtgpr_content_$name$count,
	'pr' => mtgpr_GetGooglePageRank ('$url'),
)); ?>
PHPHEREDOC

	$output;
}


;### Embed a PHP code to retrieve param;
sub prepare_content_param {
	my ($ctx, $args, $cond) = @_;
;#
	my $builder = $ctx->stash('builder');
	my $tokens  = $ctx->stash('tokens');
	defined (my $out = $builder->build ($ctx, $tokens, $cond))
		or return $ctx->error ($builder->errstr);

	$out =~ s/^\s+//;
	$out =~ s/\s+$//;
	$ctx->stash (__PACKAGE__. '_url', $out);

	;# return empty. no effective to outputs
	'';
}


;### Embed a PHP code that MTGPRContentLoop shows enclosed contens
sub content_loop {
	my ($ctx, $args) = @_;
;#
	;# retrieve arguments - name
	my $name = $args->{name} || '';
	;# retrieve arguments - count
	my $count = $args->{count} || 65536;
	;# retrieve arguments - offset
	my $offset = $args->{offset} || 0;

	;#
	my $output = $args->{sort_order} && $args->{sort_order} eq 'ascend' ?
			<<"PHPHEREDOC"
<?php \$mtgpr_sort_order = array ( '','0','1','2','3','4','5','6','7','8','9','10' ); ?>
PHPHEREDOC
		  : <<"PHPHEREDOC";
<?php \$mtgpr_sort_order = array ( '10','9','8','7','6','5','4','3','2','1','0','' ); ?>
PHPHEREDOC

	;# PHP codes
	$output .= <<"PHPHEREDOC";
<?php
\$mtgpr_item_count = 0;
foreach (\$mtgpr_sort_order as \$mtgpr_sort_order_value)
	foreach (\$mtgpr_content_${name} as \$mtgpr_content_${name}_value)
		if (\$mtgpr_content_${name}_value{pr} == \$mtgpr_sort_order_value) {
			\$mtgpr_item_count++;
			if ($offset < \$mtgpr_item_count && \$mtgpr_item_count <= $offset + $count)
				\$mtgpr_content_${name}_value{content} ();
		}
?>
PHPHEREDOC

	$output;
}

1;